﻿using PrtOn.Properties;
using System;
using System.Collections.Generic;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace PrtOn
{
    /// <summary>
    /// リスト解析と印刷画像出力
    /// </summary>
    internal class PBList
    {
        /// <summary>
        /// PBFontクラスのインスタンス
        /// </summary>
        static PBFont pbfont = new PBFont();


        /// <summary>
        /// リスト1行の変換
        /// </summary>
        /// <param name="input">入力行</param>
        /// <returns>変換後の行</returns>
        public string ConvertString(string input)
        {
            if (IsProgramArea(input))
            {
                return ConvertProgramArea(input);
            }
            else if (IsLine(input))
            {
                return ConvertLine(input);
            }
            else
            {
                return input;
            }
        }

        /// <summary>
        /// プログラムエリア判定
        /// </summary>
        /// <param name="input">入力行</param>
        /// <returns>
        /// true :プログラムエリア
        /// false:プログラムエリアではない
        /// </returns>
        static bool IsProgramArea(string input)
        {
            return Regex.IsMatch(input, @"^\[P\d{1,9}\]$");
        }

        /// <summary>
        /// プログラム行の判定
        /// </summary>
        /// <param name="input"></param>
        /// <returns>
        /// true :プログラム行
        /// false:無効は行（コメント扱い）
        /// </returns>
        static bool IsLine(string input)
        {
            return Regex.IsMatch(input, @"^\d{1,5}\s.+$");
        }

        /// <summary>
        /// プログラムエリアの変換
        /// </summary>
        /// <param name="input">入力行</param>
        /// <returns>変換後の文字列</returns>
        static string ConvertProgramArea(string input)
        {
            // 「[P」「0-9」「]」を取り除く
            return input.Replace("[P", "P").Replace("]", "");
        }

        /// <summary>
        /// 1行の変換
        /// </summary>
        /// <param name="input">入力行</param>
        /// <returns>変換後の行</returns>
        static string ConvertLine(string input)
        {
            try
            {
                // 行番号と文を分割
                string outstring = "";
                string restOfString = "";

                // 正規表現パターン: 文字列の先頭にある数字の連続部分を検出
                string pattern = @"^\d+";

                // 正規表現に基づいて行番号を取得
                Match match = Regex.Match(input, pattern);
                if (match.Success)
                {
                    string lineNumber = match.Value;
                    outstring += lineNumber.PadLeft(Settings.Default.LineNoColumn); // 行番号を右詰めで表示
                    outstring += " ";
                    restOfString = input.Substring(match.Length);
                }
                else
                {
                    restOfString = input;
                }

                // 残りの文字列を出力
                outstring += pbfont.ConvertString(restOfString.Trim());

                return outstring;
            }
            catch {
                return input;
            }
        }

        /// <summary>
        /// 16進数文字列の取得
        /// </summary>
        /// <param name="str">入力文字列</param>
        /// <returns>
        /// -1以外:変換後の値
        /// -1    :16進数文字列ではない
        /// </returns>
        private (int code, int length) ParseHexCode(string str)
        {
            // 文字列がnullまたは長さが4未満または先頭2文字が@"\x"以外の場合、-1を返す。
            if (string.IsNullOrEmpty(str) || str.Length < 4 || str.Substring(0, 2) != @"\x")
            {
                return (-1, 0);
            }

            // 2桁の16進数文字列を抽出
            string hexPart = str.Substring(2, 2);

            // 16進数として解析し、成功すればその値を返す。失敗すれば-1を返す。
            if (int.TryParse(hexPart, System.Globalization.NumberStyles.HexNumber, null, out int hexValue))
            {
                return (hexValue, 4);
            }

            return (-1, 0);
        }

        /// <summary>
        /// 16進数文字列の取得
        /// </summary>
        /// <param name="str">入力文字列</param>
        /// <returns>
        /// -1以外:変換後の値
        /// -1    :16進数文字列ではない
        /// </returns>
        private (int code, int length) ParseHexCode2(string str)
        {
            // 文字列がnullまたは長さが3未満または先頭が"\\"以外の場合-1を返す。
            if (string.IsNullOrEmpty(str) || str.Length < 3 || !str.StartsWith("\\"))
            {
                return (-1, 0);
            }

            // 2桁の16進数文字列を抽出
            string hexPart = str.Substring(1, 2);

            // 16進数として解析し、成功すればその値を返す。失敗すれば-1を返す。
            if (int.TryParse(hexPart, System.Globalization.NumberStyles.HexNumber, null, out int hexValue))
            {
                return (hexValue, 3);
            }

            return (-1, 0);
        }

        /// <summary>
        /// 文字列からPBの文字コードへの変換
        /// </summary>
        /// <param name="line">入力文字列</param>
        /// <returns>
        /// int code  : PBの文字コード、変換できなければ0
        /// int length: 変換した文字数
        /// </returns>
        public (int code, int length) ChangePBCode(string line)
        {
            int code;
            int length;

            (code, length) = ParseHexCode(line);
            if (code >= 0)
            {
                return (code, length);
            }

            (code, length) = pbfont.GetPBCode(line);
            if (code >= 0)
            {
                return (code, length);
            }

            (code, length) = ParseHexCode2(line);
            if (code >= 0)
            {
                return (code, length);
            }

            return (0, 1);
        }
    }
}
